import typer
import RegularExpressionParser.Parser as Parser
import Crawler.Scrape as Scrape
import API.Functions as API
import os
import json
import re
from dotenv import load_dotenv

app = typer.Typer()


@app.command()
def main_page():
    """
    main page of client, directly run this .py to enter and see usage
    :return: none
    """
    print("==================================================================\n\n"
          "Welcome to use goodReads scraper!\n\n"
          "This is main page\n\n"
          "You can input back to return to this page at anytime\n\n"
          "Usage:\n\n"
          "get: find a document for a book or an author from database\n\n"
          "put: create or update a document in database\n\n"
          "post: add one or multiple documents of book or author in database \n\n"
          "delete: delete certain document from database\n\n"
          "export: export a certain collection from database\n\n"
          "exit: end the program\n\n"
          "Please enter your command:\n")
    order = input()
    if order == "get":
        get()
    elif order == "put":
        put()
    elif order == "post":
        post()
    elif order == "delete":
        delete()
    elif order == "export":
        export()
    if order == "exit":
        exit()
    else:
        print("Unknown command")
    main_page()


def check_and_back(command):
    """ check if used in put back and return to main page if so """
    if command == "back":
        main_page()


def back_to_main_page():
    """ wait for user to press enter and then return to the main page """
    print("press enter to return to main page")
    no_use = input()
    main_page()


def scrape(query):
    """
    Function used to create scrape request to Server
    """
    attributes = query.split("?")[1].split("&")
    print(attributes)
    url = attributes[0].split("=", 1)[1]
    book_num = attributes[1].split("=")[1]
    if not book_num.isdigit():
        print("max book num must be an integer")
        back_to_main_page()
    book_num = int(book_num)
    author_num = attributes[2].split("=")[1]
    if not author_num.isdigit():
        print("max author num must be an integer")
        back_to_main_page()
    author_num = int(author_num)
    print('Request received, start scraping')
    if not Parser.check_if_address_valid(url):
        print("Error: invalid URL")
        back_to_main_page()
    elif book_num >= 2000 or author_num >= 500 or book_num <= 0 or author_num <= 0:
        print("Error, the number of book and author should be positive and lower than 2000/500")
        back_to_main_page()
    else:
        if book_num > 200 or author_num > 50:
            print("Warning, maximum of the number of books or authors is large")
        result = API.post(query, {})
        return result


def get():
    """ GET API """
    print("==================================================================\n")
    print("Please enter your query:\n")
    query = input()
    check_and_back(query)
    result = API.get(query)
    typer.echo(result)
    print()
    back_to_main_page()


def put():
    """ PUT API """
    print("==================================================================\n")
    print("Please enter your query:\n")
    query = input()
    check_and_back(query)
    print("Please select one way to load update info: input or read:\n")
    load = input()
    check_and_back(load)
    if load == "input":
        print("Please enter json data:\n")
        string = input()
        check_and_back(string)
        try:
            data = json.loads(string)
        except ValueError as err:
            print(err)
            back_to_main_page()
    elif load == "read":
        load_dotenv()
        file_root = os.getenv('FILE_ROOT')
        print("Please enter the name of file which is in the file root directory:\n")
        name = input()
        check_and_back(name)
        file_path = r"" + file_root + name
        if os.path.isfile(file_path):
            with open(file_path, "r") as file:
                try:
                    data = json.load(file)
                except ValueError as err:
                    print(err)
                    back_to_main_page()
        else:
            print("Error: file not exist")
            back_to_main_page()
    else:
        print("Unknown command")
        back_to_main_page()
    if re.search("books", query):
        if len(data) <= 1:
            print("For posting multiple books, please input json file has more than 1 books\n")
            main_page()
    else:
        if len(data) > 1:
            print("Cannot post more than one book with 'post book/author', please use 'post books/authors'")
            main_page()
    result = API.put(query, data)
    typer.echo(result)
    print()
    back_to_main_page()


def post():
    """ POST API """
    print("==================================================================\n")
    print("Please enter your query:\n")
    query = input()
    check_and_back(query)
    if re.search("scrape", query):
        result = scrape(query)
        typer.echo(result)
        print()
        back_to_main_page()
    else:
        load_dotenv()
        file_root = os.getenv('FILE_ROOT')
        print("Please enter the name of file which is in the file root directory:\n")
        name = input()
        check_and_back(name)
        file_path = r"" + file_root + name
        if os.path.isfile(file_path):
            with open(file_path, "r") as file:
                try:
                    data = json.load(file)
                except ValueError as err:
                    print(err)
                    back_to_main_page()
        else:
            print("Error: file not exist")
            back_to_main_page()
        result = API.post(query, data)
        typer.echo(result)
        print()
        back_to_main_page()


def delete():
    """ DELETE API """
    print("==================================================================\n")
    print("Please enter your query:\n")
    query = input()
    check_and_back(query)
    result = API.delete(query)
    typer.echo(result)
    print()
    back_to_main_page()


def export():
    """ export the book or author collection """
    print("==================================================================\n")
    print("Which collection do you want to export: books or authors?\n")
    collection = input()
    check_and_back(collection)
    if collection == "books":
        json_file = API.get("book")
        load_dotenv()
        file_root = os.getenv('FILE_ROOT')
        with open(file_root + "books" + ".json", "w") as output:
            output.write(json_file)
        print("books.json is stored at " + file_root)
        back_to_main_page()
    elif collection == "authors":
        json_file = API.get("author")
        load_dotenv()
        file_root = os.getenv('FILE_ROOT')
        with open(file_root + "authors" + ".json", "w") as output:
            output.write(json_file)
        print("authors.json is stored at " + file_root)
        back_to_main_page()
    elif collection == "back":
        back_to_main_page()
    else:
        print("Unknown collection")
        back_to_main_page()


if __name__ == "__main__":
    app()
